
import { createRoot } from 'react-dom/client'
import App from './App.tsx'
import './index.css'

// Declare window interface extension for TypeScript
declare global {
  interface Window {
    initApp?: () => void;
    appError?: Error | null;
    setPageTitle?: (title: string) => void;
    appLoaded?: boolean;
    appInstance?: any;
    appLoadTimeout?: number;
  }
}

// Function to set page title
window.setPageTitle = (title: string) => {
  document.title = title;
};

// Enhanced error logging function
const logError = (source: string, error: any) => {
  console.error(`Error in ${source}:`, error);
  
  // Create a more detailed error object for debugging
  const errorDetails = {
    message: error?.message || String(error),
    stack: error?.stack,
    timestamp: new Date().toISOString(),
    source,
    userAgent: navigator.userAgent,
    url: window.location.href
  };
  
  console.error('Error details:', errorDetails);
};

// Single instance of root to prevent multiple creation
let rootInstance: any = null;

// Function to initialize the app
const initializeApp = () => {
  // Set appLoaded to false initially
  window.appLoaded = false;
  
  // Clear timeout as app is initializing
  if (window.appLoadTimeout) {
    clearTimeout(window.appLoadTimeout);
  }

  const rootElement = document.getElementById("root");
  if (!rootElement) {
    console.error('Root element not found');
    return;
  }
  
  try {
    console.log('Initializing app...');
    window.appError = null;
    
    // Add comprehensive error handling
    const globalErrorHandler = (event: ErrorEvent) => {
      // Skip script errors without details
      if (event.message === "Script error." || event.message === "Script error") {
        console.warn('Cross-origin script error detected, continuing execution');
        event.preventDefault();
        return true;
      }
      
      // Prevent error event default handling
      event.preventDefault();
      
      logError('Global error handler', {
        message: event.message,
        filename: event.filename,
        lineno: event.lineno,
        colno: event.colno,
        error: event.error,
        isTrusted: event.isTrusted
      });
      
      window.appError = new Error(`${event.message || 'Unknown error'}`);
      
      // Only attempt recovery if we haven't tried too many times
      const recoveryAttempts = parseInt(window.sessionStorage.getItem('error_recovery_attempts') || '0');
      if (recoveryAttempts < 2) {
        window.sessionStorage.setItem('error_recovery_attempts', (recoveryAttempts + 1).toString());
        console.log(`Attempting recovery from error (attempt ${recoveryAttempts + 1})...`);
        
        // Show error UI but don't reinitialize immediately to avoid infinite loops
        showErrorUI(rootElement, event.message || 'Unknown error');
      }
      
      return true; // Prevent default error handling
    };
    
    // Remove any previous error handlers to avoid duplicates
    window.removeEventListener('error', globalErrorHandler);
    window.addEventListener('error', globalErrorHandler);
    
    // Catch unhandled promise rejections with better details
    const promiseErrorHandler = (event: PromiseRejectionEvent) => {
      logError('Unhandled promise rejection', event.reason);
      window.appError = event.reason instanceof Error 
        ? event.reason 
        : new Error(`Unhandled rejection: ${String(event.reason)}`);
      
      // Show error UI
      showErrorUI(rootElement, 
        event.reason instanceof Error ? event.reason.message : String(event.reason)
      );
    };
    
    // Remove any previous rejection handlers to avoid duplicates
    window.removeEventListener('unhandledrejection', promiseErrorHandler);
    window.addEventListener('unhandledrejection', promiseErrorHandler);
    
    // Create a safer render method with error boundary
    const renderApp = () => {
      try {
        // Only create root once
        if (!rootInstance) {
          rootInstance = createRoot(rootElement);
        }
        
        rootInstance.render(<App />);
        console.log('App initialized successfully');
        window.appLoaded = true;
        window.appInstance = rootInstance;
      } catch (error) {
        logError('React render', error);
        // Add a fallback UI for critical render errors
        showErrorUI(rootElement, error instanceof Error ? error.message : String(error));
      }
    };
    
    // Mark app as loaded when component is actually rendered
    renderApp();
  } catch (error) {
    logError('App initialization', error);
    window.appError = error instanceof Error ? error : new Error(String(error));
    
    // Add fallback UI for critical initialization errors
    showErrorUI(rootElement, error instanceof Error ? error.message : String(error));
  }
};

// Helper function to show error UI
const showErrorUI = (rootElement: HTMLElement, errorMessage: string) => {
  rootElement.innerHTML = `
    <div style="padding: 20px; text-align: center;">
      <h2>Something went wrong</h2>
      <p>${errorMessage || 'An unknown error occurred'}</p>
      <button 
        onclick="window.location.reload()" 
        style="padding: 8px 16px; background: #4f46e5; color: white; border: none; border-radius: 4px; cursor: pointer; margin-top: 16px;">
        Refresh Page
      </button>
    </div>
  `;
};

// Initialize the app immediately
initializeApp();

// Make the app initialization available for embedding
window.initApp = initializeApp;
